const htmlparser = require("htmlparser2");
const recast = require('recast');
const Property = require('./model/Property');
const Event = require('./model/Event');
const Component = require('./model/Component');


class VueComponentAnalyzer {
    constructor(params) {
        this.domScript = params.domScript;
        this.javascript = params.javascript;
        this.componentName = params.componentName;
        this.events = [];
        this.tagNames = [];
        this.props = [];
        this._analyzeDomScript();
        this._analyzeJavascript();
        this.component = new Component(this.componentName,this.props,this.events,this.tagNames);
    }

    getComponent(){
        return this.component;
    }

    /**
     *  解析dom脚本
     */
    _analyzeDomScript() {
        if (!this.domScript) return;
        let parser = new htmlparser.Parser({
            onopentag: (tagName, attribs)=>{
                this._addTagName(tagName);
                if(attribs){
                    for(let [attrName,attrVal] of  Object.entries(attribs)){
                        if (attrName.startsWith("@") || attrName.startsWith("v-on")) {
                            if (attrVal.indexOf("$emit") != -1) {
                                /* 解析emit */
                                let event = this._parseDomEventCaller(attrVal);
                                if (event) {
                                    this._addEvent(event);
                                }
                            }
                        }
                    }
                }
            }
        }, {decodeEntities: true,lowerCaseTags:false});
        parser.write(this.domScript);
        parser.end();
    }

    _addTagName(tagName){
        if(this.tagNames.indexOf(tagName)==-1){
            this.tagNames.push(tagName);
        }
    }

    _addEvent(event) {
        let notFound = true;
        this.events.forEach(evt => {
            if (evt.getEventCode() == event.getEventCode()) {
                notFound = false;
                if (evt.getArgumentNum() < event.getArgumentNum()) {
                    evt.setArgumentNum(event.getArgumentNum());
                }
            }
        });
        if (notFound) {
            this.events.push(event);
        }
    }

    /**
     * 解析emit字符串
     * @param {String} val emit字符串
     */
    _parseDomEventCaller(val) {
        try {
            let ast = recast.parse(val);
            let events = [];
            recast.visit(ast, {
                events: events,
                visitExpressionStatement: function (path) {
                    let node = path.value;
                    let exp = node.expression;
                    let callee = exp.callee;
                    if (callee) {
                        if (callee.name == "$emit") {
                            let args = exp.arguments;
                            if (args.length > 0) {
                                let eventCode = args[0].value;
                                this.visitor.events.push({
                                    eventCode: eventCode,
                                    argumentNum: args.length - 1
                                });
                            } else {
                                throw Error("解析事件参数错误，没有配置事件名称！");
                            }
                        }
                    }
                    this.traverse(path);
                }
            });
            if (events.length > 0) {
                let evt = events[0];
                return new Event(evt.eventCode, evt.argumentNum);
            } else {
                return null;
            }
        } catch (e) {
            throw new Error("解析事件参数错误，解析的字符串：[" + val + "]");
        }
    }

    _analyzeJavascript() {
        if (!this.javascript) return;
        let ast = recast.parse(this.javascript);
        let infos = {
            events: [],
            props: []
        };
        recast.visit(ast, {
            infos: infos,
            visitProperty: function (path) {
                var node = path.value;
                if (node.key && node.key.name == "props" && path.parent && path.parent.name == "declaration" && path.parent.value.type == "ObjectExpression") {
                    let value = node.value;
                    if (value.type != "Identifier") {
                        let code = recast.print(value).code;
                        let props = eval("(" + code + ")");
                        for (let [name, val] of Object.entries(props)) {
                            let prop = {
                                name: name
                            };
                            let valType = typeof (val);
                            switch (valType) {
                                case 'function':
                                    prop.type = val.name;
                                    break;
                                case 'object':
                                    if (Array.isArray(val)) {
                                        let types = [];
                                        val.forEach((item) => {
                                            if (typeof (item) == 'function') {
                                                types.push(item.name);
                                            } else {
                                                types.push(item);
                                            }
                                        });
                                        prop.type = types;
                                    } else if (Object(val) === val) {
                                        for (let [propName, propVal] of Object.entries(val)) {
                                            if (propName == "type") {
                                                if (typeof (propVal) == "function") {
                                                    prop.type = propVal.name;
                                                } else if (Object(propVal) !== propVal) {
                                                    let types = [];
                                                    propVal.forEach((item) => {
                                                        if (typeof (item) == 'function') {
                                                            types.push(item.name);
                                                        } else {
                                                            types.push(item);
                                                        }
                                                    });
                                                    prop.type = types;
                                                }
                                            } else if (propName == "required") {
                                                prop.required = propVal;
                                            } else if (propName == "default") {
                                                prop.default = propVal;
                                            }
                                        }
                                    }
                                    break;
                                default:

                            }
                            this.visitor.infos.props.push(new Property(prop.name, prop.type, prop.default,prop.required));
                        }
                    }
                }
                this.traverse(path);
            },
            visitExpressionStatement: function (path) {
                let node = path.value;
                let exp = node.expression;
                let callee = exp.callee;
                if (callee) {
                    let obj = callee.object;
                    let property = callee.property;
                    if (obj && property && obj.type == "ThisExpression" && property.type == "Identifier" && property.name == "$emit") {
                        let args = exp.arguments;
                        let eventCode = args[0].value;
                        this.visitor.infos.events.push(new Event(eventCode, args.length - 1));
                    }
                }
                this.traverse(path);
            }
        });
        if (infos.events.length > 0) {
            infos.events.forEach(event => {
                this._addEvent(event);
            });
        }
        this.props = infos.props;
    }
}

module.exports = VueComponentAnalyzer;