/**
 * @author xdh
 * @modify  xujw 2018-1-31
 * install命令执行任务,调用对应的解析器,解析js脚本,找出所有用户自定义的vui组件
 */
const ITask = require("../ITask");
const pathUtils = require("../../../../utils/PathUtils");
const fileUtils = require("../../../../utils/FileUtils");
const objectUtils = require('../../../../utils/ObjectUtils');
const V3PlatformJar = require("../../model/V3PlatformJar");
const fs = require("fs");
const path = require('path');
const configUtils = require('../../../../utils/ConfigUtils');
const widgetPathUtils = require('../../../utils/WidgetPathUtils');
const VueComponentFileAnalyzer = require('../../../../analyzer/VueComponentFileAnalyzer');

class InstallTask extends ITask {

	constructor(params) {
		super(params);
		this.engineContext = null;
		this.resolveFn = null;
		this.rejectFn = null;
		this.libs = null;
	}

	exe(ctx) {
		this.engineContext = ctx;
		return new Promise((resolve, reject) => {
			this.resolveFn = resolve;
			this.rejectFn = reject;
			try {
				if (this.validateCfg()) {
					let promise = this.search();
					promise.then(res=>{
						this.libs = res;
						let pluginCode = configUtils.getPluginCode();
						let type = configUtils.getV3DevConfig().type;
						if (type == 'resource') {
							this.generateJar();
						} else {
							/* 创建vue组件解析器 */
							let analyzer = new VueComponentFileAnalyzer(pluginCode, widgetPathUtils.getWidgetVuePath());
							this.generateJar(analyzer);
						}
					}).catch(err=>{
						this.rejectFn(err);
					});
				}
			} catch (err) {
				reject(err);
			}
		});
	}

	generateJar(analyzer) {
		let dependencies = [];
		let v3DevCfg = configUtils.getV3DevConfig();
		if (v3DevCfg && v3DevCfg.dependencies) {
			v3DevCfg.dependencies.forEach(dep => {
				dependencies.push(dep.name);
			});
		}
		var _libs = this.libs;
		let jar = new V3PlatformJar({
			"packageJson": configUtils.getPackageConfig(),
			"v3DevConfig": v3DevCfg,
			"libs": _libs,
			"components": analyzer ? [analyzer.getComponent()] : [],
			"version": configUtils.getPluginVersion(),
			// "tarballPath": this.engineContext.get("tgzPath"),
			"dependencies": dependencies,
			"context": this.engineContext
		});
		jar._init().then(() => {
			var resourcePaths = [];
			if (_libs && _libs.length > 0) {
				for (let lib of _libs) {
					resourcePaths.push(path.relative(pathUtils.getPluginPath(), lib.path).replace(/\\/g, '/'));
				}
			}
			/* 项配置文件中添加libs属性 */
			v3DevCfg.libs = resourcePaths;
			this.engineContext.put("jar", jar);
			let promise = jar.persistence();
			promise.then(archive => {
				configUtils.persistenceV3DevCfg();
				// fs.unlinkSync(this.engineContext.get("tgzPath"));
				fs.unlinkSync(jar.tarballPath);
				console.log('VStore构件生成完成，总共：' + archive.pointer() + "字节！");
				this.resolveFn(jar);
			}).catch(err => {
				configUtils.persistenceV3DevCfg();
				this.rejectFn(err);
			});
		}).catch(err => {
			this.rejectFn(err);
		});
	}

	//查找webpack打包生成的资源
	searchWebpackedResource() {
		let sources = configUtils.getPluginResources();
		sources = objectUtils.isArray(sources) ? sources : [sources];
		let libs = [];
		let pluginType = configUtils.getPluginType();
		for (let i = 0, l = sources.length; i < l; i++) {
			let pluginPath = pathUtils.toPluginPath(path.join(sources[i], pluginType));
			if (fs.existsSync(pluginPath)) {
				/* lib是文件且不为正则表达式所匹配的文件类型 */
				if (fileUtils.isFile(pluginPath)) {
					libs.push({
						path : pluginPath,
						cached : true
					});
				} else { //文件夹
					let paths = fileUtils.listFiles(pluginPath, true);
					for (let j = 0, len = paths.length; j < len; j++) {
						let tmPath = paths[j];
						if (fileUtils.isFile(tmPath)) {
							libs.push({
								path : tmPath,
								cached : true
							});
						}
					}
				}
			} else {
				console.warn(`WARN:未查找到webpack打包后的资源，资源路径：${pluginPath}`);
			}
		}
		return libs;
	}

	//搜索额外添加的资源
	searchExtraSources() {
		return new Promise((resolve,reject)=>{
			try{
				let extraSources = configUtils.getPluginExtraSources();
				let libs = [];
				if (extraSources) {
					extraSources = objectUtils.isArray(extraSources) ? extraSources : [extraSources];
					for (let extraRes of extraSources) {
						let cache = true,path;
						if(typeof(extraRes)=='string'){
							path = extraRes;
						}else{
							path = extraRes.path;
							cache = extraRes.hasOwnProperty("cached") ? extraRes.cached:cache;
						}
						let resPath = pathUtils.toPluginPath(path);
						if (fs.existsSync(resPath)) {
							if (fileUtils.isFile(resPath)) {
								libs.push({
									path : resPath,
									cached : cache
								});
							} else { //文件夹
								let paths = fileUtils.listFiles(resPath, true);
								for (let j = 0, len = paths.length; j < len; j++) {
									let tmPath = paths[j];
									if (fileUtils.isFile(tmPath)) {
										libs.push({
											path : tmPath,
											cached : cache
										});
									}
								}
							}
						} else {
							this.rejectFn(new Error(`ERROR: .v3devrc文件中extraResources配置有误，文件或文件夹不存在！path=${resPath}`));
						}
					}
				}
				resolve(libs);
			}catch(err){
				reject(err);
			}	
		});
	}

	search() {
		return new Promise((resolve,reject)=>{
			let promise = this.searchExtraSources();
			promise.then(res=>{
				let webpackedResources = this.searchWebpackedResource();
				res = res.concat(webpackedResources);
				resolve(res);
			}).catch(err=>{
				reject(err);
			});
		});
	}
	/**
	 * 校验配置信息
	 */
	validateCfg() {
		try {
			let sources = configUtils.getPluginResources();
			if (sources) {
				return true;
			} else {
				this.rejectFn(new Error("ERROR: .v3devrc文件中没有配置sources属性，v3平台打包将终止！"));
			}
			return false;
		} catch (err) {
			this.rejectFn(err);
		}
	}

	handleError(ctx) {
		return new Promise((resolve, reject) => {
			try {
				var jar = ctx.get("jar");
				if (jar) {
					var dist = jar.getDist();
					if (fs.existsSync(dist)) {
						fs.unlinkSync(dist);
					}
					var sourceDist = jar.getSource();
					if (fs.existsSync(sourceDist)) {
						fs.unlinkSync(sourceDist);
					}
				}
				resolve();
			} catch (err) {
				reject(err);
			}
		});
	}

	logResources(libs) {
		console.log("检索插件资源完成，检索到的资源如下：");
		for (var i = 0, l = libs.length; i < l; i++) {
			console.log(libs[i]);
		}
	}
}

module.exports = InstallTask;