const pathUtils = require('./PathUtils');
const path = require('path');
const PathSep = path.sep;
const fs = require('fs');
const prompt = require('inquirer').prompt;
const stringUtils = require('./StringUtils');
const Constant = require('./Constant');
const ioUtils = require('./IOUtils');
const mavenUtils = require('./MavenUtils');
const cliUtils = require('v3-cli/src/utils/UpdateUtils');
let configs = null;
let selfConfigs = null;
let packageJson = null;
let selfPackageJson = null;
const v3CfgName = '.v3devrc';

module.exports = {
    // 设置当前测试服务环境变量
    setTestServerEnv: function (env) {
        process.env.V3CLI_DEV_SERVER = env;
    },
    // 获取当前环境是否要启用vtheme-loader
    enableThemeLoader: function () {
        if (process.env.V3CLI_DEV_SERVER) {
            return false;
        }
        let cliProperty = cliUtils.getProperty();
        if (!cliProperty.vthemeSupport) {
            return false;
        }
        return true;
    },
    /**
     * @deprecated
     */
    getNodeModuleV3DevConfigs: function () {
        let configs = [];
        let modulePath = path.resolve(pathUtils.getPluginPath(), 'node_modules');
        if (!fs.existsSync(modulePath)) {
            throw Error("文件路径不存在，请检查！path=" + modulePath);
        }
        let fileNames = fs.readdirSync(modulePath);
        if (fileNames && fileNames.length > 0) {
            fileNames.forEach(fileName => {
                let absPath = path.resolve(modulePath, fileName, v3CfgName);
                if (fs.existsSync(absPath)) {
                    let content = fs.readFileSync(absPath);
                    configs.push(JSON.parse(content));
                }
            });
        }
        return configs;
    },
    /**
     * @deprecated
     */
    getNodeModulePluginJsons: function () {
        let configs = [];
        let modulePath = path.resolve(pathUtils.getPluginPath(), 'node_modules');
        if (!fs.existsSync(modulePath)) {
            throw Error("文件路径不存在，请检查！path=" + modulePath);
        }
        let fileNames = fs.readdirSync(modulePath);
        if (fileNames && fileNames.length > 0) {
            fileNames.forEach(fileName => {
                let absPath = path.resolve(modulePath, fileName, v3CfgName);
                if (fs.existsSync(absPath)) {
                    let packageJsonPath = path.resolve(modulePath, fileName, 'package.json');
                    let content = fs.readFileSync(packageJsonPath);
                    configs.push(JSON.parse(content));
                }
            });
        }
        return configs;
    },
    /**
     * 获取用户项目的配置信息
     */
    getV3DevConfig: function () {
        if (!configs) {
            let v3devCfgPath = pathUtils.toPluginPath(v3CfgName);
            if (fs.existsSync(v3devCfgPath)) {
                let content = fs.readFileSync(v3devCfgPath);
                configs = JSON.parse(content);
            } else {
                throw Error("当前插件不存在.v3devrc配置文件，请运行v3:init命令生成！");
            }
        }
        return configs;
    },
    /**
     * 通过绝对路径获取v3配置文件信息
     */
    getV3CfgByAbs: function (absPath) {
        if (!path.isAbsolute(absPath) || !fs.existsSync(absPath)) {
            return null;
        }
        let content = fs.readFileSync(absPath);
        return JSON.parse(content);
    },
    getSelfV3DevConfig: function () {
        if (!selfConfigs) {
            let selfV3devCfgPath = path.join(pathUtils.getSelfPath(), v3CfgName);
            if (fs.existsSync(selfV3devCfgPath)) {
                let content = fs.readFileSync(selfV3devCfgPath);
                selfConfigs = JSON.parse(content);
            } else {
                throw Error("当前插件不存在.v3devrc配置文件，请运行v3:init命令生成！");
            }
        }
        return selfConfigs;
    },

    /**
     * 获取插件资源目录
     */
    getPluginResources: function () {
        let config = this.getV3DevConfig();
        return config.sources;
    },

    /**
     * 获取额外的插件资源
     */
    getPluginExtraSources: function () {
        let config = this.getV3DevConfig();
        return config.extraSources || [];
    },

    /**
     * 获取插件类型
     */
    getPluginType: function () {
        let config = this.getV3DevConfig();
        return config.type;
    },
    /**
     * 验证信息
     */
    getVerification: function () {
        return new Promise((resolve, reject) => {
            let result = {};
            let config = this.getV3DevConfig();
            let questions = [];
            if (stringUtils.isEmpty(config.account)) {
                questions.push({
                    message: 'VStore账号:',
                    default: null,
                    name: 'account',
                    validate: function (input) {
                        if (stringUtils.isEmpty(input)) {
                            return "请输入VStore账号";
                        } else {
                            return true;
                        }
                    }
                });
            }
            if (stringUtils.isEmpty(config.pwd)) {
                questions.push({
                    type: 'password',
                    message: 'Vstore密码:',
                    default: null,
                    name: 'pwd',
                    validate: function (input) {
                        if (stringUtils.isEmpty(input)) {
                            return "请输入Vstore密码";
                        } else {
                            return true;
                        }
                    }
                });
            }
            if (questions.length > 0) {
                prompt(questions).then(answers => {
                    if (!stringUtils.isEmpty(answers.account)) {
                        config.account = answers.account;
                    }
                    if (!stringUtils.isEmpty(answers.pwd)) {
                        config.pwd = answers.pwd;
                    }
                    this.persistenceV3DevCfg();
                    resolve({
                        account: config.account,
                        pwd: config.pwd
                    });
                });
            } else {
                resolve({
                    account: config.account,
                    pwd: config.pwd
                });
            }
        });
    },
    /* 插件依赖去重 */
    depRepeatRvm(dependencies) {
        /* 为空或者小于等于1都不需要去重 */
        if (!dependencies || dependencies.length <= 1) {
            return dependencies;
        }
        let hash = {};
        dependencies = dependencies.reduce((item, next) => {
            if (hash[next.name]) {
                if (next.version && hash[next.name].version && next.version > hash[next.name].version) {
                    hash[next.name] = next;
                    item = item.map(dep => {
                        if (dep.name == next.name) {
                            dep = next;
                        }
                        return dep;
                    });
                }
            } else {
                hash[next.name] = next;
                item.push(next);
            }
            return item;
        }, []);
        return dependencies;
    },
    persistenceV3DevCfg: function () {
        if (configs) {
            /* 保存前去除重复的vjs依赖项 */
            configs.VjsList = this.depRepeatRvm(configs.VjsList);
            configs.depVjsList = this.depRepeatRvm(configs.depVjsList);
            configs.dependencies = this.depRepeatRvm(configs.dependencies);
            configs.require = this.depRepeatRvm(configs.require);
            ioUtils.writeContent(pathUtils.toPluginPath(v3CfgName), JSON.stringify(configs, null, '\t'));
        }
    },
    setNewV3DevCfg: function (newCfg) {
        /* 设置新的配置信息 */
        configs = newCfg;
    },
    /**
     * 获取执行系统地址
     */
    getServer: function () {
        return new Promise((resolve, reject) => {
            let config = this.getV3DevConfig();
            if (stringUtils.isEmpty(config.server)) {
                prompt([{
                    message: '执行系统地址:',
                    default: null,
                    name: 'server',
                    validate: function (input) {
                        if (stringUtils.isEmpty(input)) {
                            return "请输入执行系统地址";
                        } else {
                            return true;
                        }
                    }
                }]).then(answers => {
                    config.server = answers.server;
                    this.persistenceV3DevCfg();
                    resolve({
                        server: config.server
                    });
                });
            } else {
                resolve({
                    server: config.server
                });
            }
        });
    },

    /**
     * 获取插件编号
     */
    getPluginCode: function () {
        let config = this.getV3DevConfig();
        return config.pluginCode;
    },

    /**
     * 获取配置文件路径
     */
    getV3DevConfigPath: function () {
        return pathUtils.toPluginPath(v3CfgName);
    },

    /**
     * 获取package.json配置信息
     */
    getPackageConfig: function () {
        if (!packageJson) {
            let packagePath = pathUtils.toPluginPath("package.json");
            packageJson = require(packagePath);
        }
        return packageJson;
    },

    getSelfPackageConfig: function () {
        if (!selfPackageJson) {
            let packagePath = path.join(pathUtils.getSelfPath(), 'package.json');
            selfPackageJson = require(packagePath);
        }
        return selfPackageJson;
    },

    /**
     * 获取插件版本
     */
    getPluginVersion: function () {
        let config = this.getV3DevConfig();
        return config.version;
    },
    /**
     * 通过依赖名称获取依赖的node目录路径
     */
    getDepNodeDir: function (dependenceName) {
        let nodeProjectName = dependenceName.substring(dependenceName.lastIndexOf('.') + 1);
        return path.resolve(pathUtils.getPluginPath(), 'node_modules', nodeProjectName);
    },
    /** 获取带插件类型的完整symbolicName
     * @param name 不带插件类型的symbolicName
     */
    getSymNameWithType: function (name, type) {
        /* 由于依赖信息中的name属性不完整,可能缺少插件类型,会导致symbolicName与Vstore中的symbolicName不匹配因此手动添加 */
        let subStr = name.substring(0, name.lastIndexOf('.'));
        let pluginType = subStr.substring(subStr.lastIndexOf('.') + 1);
        let symbolicName;
        /* 如果pluginType解析到custom说明symbolicName中没有插件类型信息,需要手动补充 */
        if (pluginType == 'custom') {
            let pluginName = name.substring(name.lastIndexOf('.'));
            symbolicName = subStr + type ? '.' + type : '.widget' + pluginName;
        } else {
            symbolicName = name;
        }
        return symbolicName;
    },
    /* 通过vjs名称推测SymName */
    getSymNameByVjs: function (vjsName, type) {
        let vjsArr = vjsName.split(".");
        let symbolicName = mavenUtils.getSymbolicName(Constant.DEFAULT_GROUP_ID, type || 'widget', vjsArr[vjsArr.length - 1]);
        return symbolicName;
    },
    /* 获取用户项目的全部依赖插件的配置文件 */
    getAllDepV3Rc: function () {
        let depV3rc = {};
        let userCfg = this.getV3DevConfig();
        let deps = userCfg.dependencies.concat(userCfg.require);
        deps.forEach(dep => {
            let path = this.getDepNodeDir(dep.name) + PathSep + v3CfgName;
            if (fs.existsSync(path)) {
                depV3rc[dep.name] = JSON.parse(fs.readFileSync(path));
            }
        });
        return depV3rc;
    },
    /* 传入指定依赖数组,以及要删除的依赖名称,返回删除后的依赖数组对象*/
    depDeleteByName: function (dependencies, deleteName) {
        if (!dependencies && dependencies.length == 0) {
            return [];
        }
        for (let i = 0; i < dependencies.length; i++) {
            if (dependencies[i].name == deleteName) {
                dependencies.splice(i, 1);
            }
        }
        return dependencies;
    },
    /* 传入指定VJS依赖数组,以及要删除的VJS依赖所属SymbolicName,返回删除后的VJS依赖数组对象*/
    VjsDeleteBySymbolicName: function (vjsList, symbolicName) {
        if (!vjsList || vjsList.length == 0) {
            return [];
        }
        for (let i = 0; i < vjsList.length; i++) {
            if (vjsList[i].symbolicName && vjsList[i].symbolicName == symbolicName) {
                vjsList.splice(i, 1);
            } else if (this.getSymNameByVjs(vjsList[i].name) == symbolicName) {
                vjsList.splice(i, 1);
            }
        }
        return vjsList;
    },
    /**
     * 获取插件的urlsorces并分类返回
     */
    getUrlSourcesByCfg: function (v3cfg) {
        let urlSources = {
            js: [],
            css: []
        }
        if (v3cfg && v3cfg.urlSources) {
            v3cfg.urlSources.forEach(urlSource => {
                if (typeof urlSource == 'string') {
                    let checkurl = urlSource;
                    let index = urlSource.indexOf('?');
                    if (index > 0) {
                        checkurl = url.substring(0, index);
                    }
                    if (checkurl.endsWith('.js')) {
                        urlSources.js.push(urlSource);
                    } else if (checkurl.endsWith('.css')) {
                        urlSources.css.push(urlSource);
                    } else {
                        throw new Error(`插件${v3cfg.pluginCode}的urlSources配置【${urlSource}】错误,无.js或.css标识的链接请使用对象方式配置!示例:{"url":"${urlSource}","type":"js/css"}`);
                    }
                } else if (typeof urlSource == 'object') {
                    if (!urlSource.type) {
                        throw new Error(`插件${v3cfg.pluginCode}的urlSources配置【${urlSource}】错误,请指定资源链接type属性!`);
                    }
                    if (urlSource.type == 'js') {
                        urlSources.js.push(urlSource.url);
                    } else if (urlSource.type == 'css') {
                        urlSources.css.push(urlSource.url);
                    } else {
                        throw new Error(`插件${v3cfg.pluginCode}的urlSources配置【${urlSource}】错误,暂不支持的type属性:${urlSource.type}!`);
                    }
                }
            });
        }
        return urlSources;
    }
};